@@ -28,8 +28,8 @@ def all_subclasses(cls):
     """ Recursively generate of all the subclasses of class cls. """
     for subclass in cls.__subclasses__():
         yield subclass
-        for subclass in all_subclasses(subclass):
-            yield subclass
+        for subc in all_subclasses(subclass):
+            yield subc
 
 
 def unique_justseen(iterable, key=None):
